@@ -25,23 +25,6 @@ class BIMNode (GeometryNode):
     - Convert non-RDF metadata files (.ifc, .obj, ect..) to BIMNodes and export them to RDF
     - use URIRef() to reference the node, ect...
     """
-    # class attributes
-    # ifcPath = None # (string) path to an ifc file
-    # globalId=None # (string) ifc guid of the element e.g. 2DhaSlBmz2puIzqaWbJt0S
-    # className=None #(string) ifcElement class e.g. ifcWall
-    # mesh = None # (o3d.mesh) mesh geometry of the ifcElement (MB)
-    # ifcElement = None # (ifcopenshell.entity_instance)  Don't store an ifcElement in your node. it can't be properly passed around
-    # self.path = None # (string) path of the o3d.mesh geometry
-    # self.ifc = None # (string) path to an ifc file => you don't want to store the entire ifc file!
-    # self.ifcElement = None # (string) 
-    # self.pcds = None #(List[URIRef]) => link to other pcd. 
-    
-    #Classification
-    # self.label = None #(string) ??? (MB)
-    # self.meshes = None # (List[URIRef]) => link to other meshes. (MB)
-    
-    #Geometry
-    
     # # Monitoring
     # self.phase = None # (string) ??? this can't be really extracted from the IFC (MB)
     # self.progress = None # (double [0.0;1.0]) percentage of completion
@@ -85,13 +68,14 @@ class BIMNode (GeometryNode):
             A BIMNode with metadata (if you also want the data, call node.get_resource() method)
         """   
         # set the graph and optional parameters
+        
         super().__init__(   graph= graph,
                             graphPath= graphPath,
                             subject= subject,
-                            kwargs = kwargs) 
+                            **kwargs) 
         self.path=path
         self.mesh=mesh
-        self.ifcName = None
+        self.name = None
         self.ifcPath=ifcPath
         self.globalId=globalId
         self.className=className
@@ -177,7 +161,7 @@ class BIMNode (GeometryNode):
                 return True
         return False
  
-    def export_geometry (self, extension = '.ply') ->bool:
+    def export_resource(self, directory:str=None,extension :str = '.ply') ->bool:
         """_summary_
 
         Args:
@@ -194,39 +178,33 @@ class BIMNode (GeometryNode):
             if getattr(self,'path',None) is not None and os.path.exists(self.path):
                 pass
             else:
-                #check name
-                if getattr(self,'name',None) is not None:
-                    self.name=ut.check_string_validity(self.name)
-                else:
-                    self.name=self.guid
 
-                #check session/graphPath and directory
-                if getattr(self,'sessionPath',None) is not None :
-                    if os.path.exists((self.sessionPath +'\\BIM') ):
-                        dir=self.sessionPath +'\\BIM'
-                    else:
-                        dir=os.mkdir((self.sessionPath+'\\BIM'))
-                elif getattr(self,'graphPath',None) is not None: 
-                    if os.path.exists(self.graphPath +'\\BIM' ):
-                        dir=self.graphPath +'\\BIM'
-                    else:
-                        dir=os.mkdir((self.graphPath+'\\BIM'))                    
+                #check directory
+                if directory is not None:
+                    if not os.path.exists(directory):                        
+                        os.mkdir(directory)
                 else:
-                    if os.path.exists((os.getcwd()+'\\BIM' )):
-                        dir=os.getcwd()+'\\BIM'
+                    if getattr(self,'graphPath',None) is not None: 
+                        if os.path.exists(self.graphPath +'\\BIM' ):
+                            directory=self.graphPath +'\\BIM'
+                        else:
+                            directory=os.mkdir((self.graphPath+'\\BIM'))                    
                     else:
-                        dir=os.mkdir((os.getcwd()+'\\BIM'))
-                                    
+                        if os.path.exists((os.getcwd()+'\\BIM' )):
+                            directory=os.getcwd()+'\\BIM'
+                        else:
+                            directory=os.mkdir((os.getcwd()+'\\BIM'))
+                                   
                 #check extension
                 if extension == '.ply' or extension == '.obj':
-                    self.path=dir+'\\'+self.name+extension
+                    self.path=directory+'\\'+self.subject+extension
                 else:
                     raise ValueError ('only .ply or .obj allowed') 
             try:
                 o3d.io.write_triangle_mesh(self.path, self.mesh)
                 return True
             except:
-                print("Export failed of " + self.name )
+                print("Export failed of " + self.subject )
         return False
 
     def get_metadata_from_mesh(self) -> bool:
@@ -249,28 +227,28 @@ class BIMNode (GeometryNode):
             print('No proper geometries found to extract the metadata. len(self.mesh.triangles) >1')
             return False
     
-    def get_metadata_from_ifc(self, getGeometry = False) -> bool:
+    def get_metadata_from_ifc(self, getResource : bool = False) -> bool:
         try:
             ifc = ifcopenshell.open(self.ifcPath)   
             ifcElement= ifc.by_guid(self.globalId)
-            self.ifcName=ifcElement.Name 
-            if getattr(self,'ifcName',None) is not None:
-                self.name=ut.check_string_validity(self.ifcName)
-            self.className=ifcElement.is_a()
-            if getGeometry:
+            self.name=ifcElement.Name 
+            if getattr(self,'name',None) is not None:
+                self.subject=ut.check_string_validity(self.name)
+            self.className=ifcElement.is_a()        
+            if getResource:
                 self.mesh=gt.ifc_to_mesh(ifcElement)
         except:
             print('IFC error')
             return False
 
-    def get_metadata_from_ifcElement(self,ifcElement:ifcopenshell.entity_instance, getGeometry = False) -> bool:
+    def get_metadata_from_ifcElement(self,ifcElement:ifcopenshell.entity_instance, getResource : bool = False) -> bool:
         try:  
-            self.ifcName=ifcElement.Name
-            if getattr(self,'ifcName',None) is not None:
-                self.name=ut.check_string_validity(self.ifcName)
+            self.name=ifcElement.Name
+            if getattr(self,'name',None) is not None:
+                self.subject=ut.check_string_validity(self.name)
             self.className=ifcElement.is_a()
             self.globalId=ifcElement.GlobalId
-            if getGeometry:
+            if getResource:
                 self.mesh=gt.ifc_to_mesh(ifcElement)
         except:
             print('ifcElement error')
