@@ -28,7 +28,7 @@ class GeometryNode (Node):
         super().__init__(   graph= graph,
                             graphPath= graphPath,
                             subject= subject,
-                            kwargs = kwargs) 
+                            **kwargs) 
         self.cartesianBounds=None     # (nparray [6x1]) [xMin,xMax,yMin,yMax,zMin,zMax]
         self.orientedBounds=None     # (nparray [6x1]) [xMin,xMax,yMin,yMax,zMin,zMax]
         self.oriendtedBoundingBox=None # (open3d.geometry.OrientedBoundingBox) 
