@@ -10,6 +10,7 @@ import pye57
 import xml.etree.ElementTree as ET 
 import math
 from typing import List
+import matplotlib
 
 import ifcopenshell
 import ifcopenshell.geom as geom
@@ -20,6 +21,9 @@ from scipy.spatial.transform import Rotation as R
 import geomapi
 import geomapi.utils as ut
 
+
+#### COORDINATE CONVERTERS ####
+
 def belgian_datum_to_wgs84(latBel:float,lngBel:float) -> tuple:
     """
     Input parameters : Lat, Lng : latitude / longitude in decimal degrees and in Belgian 1972 datum
@@ -65,6 +69,131 @@ def belgian_datum_to_wgs84(latBel:float,lngBel:float) -> tuple:
     LngWGS84 = ((Lng + DLng) * 180) / math.PI
     return LatWGS84,LngWGS84
 
+def wgs84_to_belgian_datum(LatWGS84: float,LngWGS84:float) -> tuple:
+    """
+    Input parameters : Lat, Lng : latitude / longitude in decimal degrees and in WGS84 datum
+    Output parameters : LatBel, LngBel : latitude / longitude in decimal degrees and in Belgian datum
+    source: http://zoologie.umons.ac.be/tc/algorithms.aspx
+    """
+    Haut = 0    
+    
+    #conversion to radians
+    Lat = (math.PI / 180) * LatWGS84
+    Lng = (math.PI / 180) * LngWGS84
+    
+    SinLat = math.Sin(Lat)
+    SinLng = math.Sin(Lng)
+    CoSinLat = math.Cos(Lat)
+    CoSinLng = math.Cos(Lng)
+    
+    dx = 125.8
+    dy = -79.9
+    dz = 100.5
+    da = 251.0
+    df = 0.000014192702
+    
+    LWf = 1 / 297
+    LWa = 6378388
+    LWb = (1 - LWf) * LWa
+    LWe2 = (2 * LWf) - (LWf * LWf)
+    Adb = 1 / (1 - LWf)
+    
+    Rn = LWa / math.Sqrt(1 - LWe2 * SinLat * SinLat)
+    Rm = LWa * (1 - LWe2) / (1 - LWe2 * Lat * Lat) ^ 1.5
+    
+    DLat = -dx * SinLat * CoSinLng - dy * SinLat * SinLng + dz * CoSinLat
+    DLat = DLat + da * (Rn * LWe2 * SinLat * CoSinLat) / LWa
+    DLat = DLat + df * (Rm * Adb + Rn / Adb) * SinLat * CoSinLat
+    DLat = DLat / (Rm + Haut)
+    
+    DLng = (-dx * SinLng + dy * CoSinLng) / ((Rn + Haut) * CoSinLat)
+    Dh = dx * CoSinLat * CoSinLng + dy * CoSinLat * SinLng + dz * SinLat
+    Dh = Dh - da * LWa / Rn + df * Rn * Lat * Lat / Adb
+    
+    LatBel = ((Lat + DLat) * 180) / math.PI
+    LngBel = ((Lng + DLng) * 180) / math.PI
+    return  LatBel,LngBel
+
+def spherical_coordinates_to_lambert72(latBel :float,lngBel:float) -> tuple:
+    """
+    Conversion from spherical coordinates to Lambert 72
+    Input parameters : lat, lng (spherical coordinates)
+    Spherical coordinates are in decimal degrees converted to Belgium datum!
+    source: http://zoologie.umons.ac.be/tc/algorithms.aspx
+    """
+ 
+    LongRef  = 0.076042943  #      '=4°21'24"983
+    bLamb  = 6378388 * (1 - (1 / 297))
+    aCarre  = 6378388 ^ 2
+    eCarre = (aCarre - bLamb ^ 2) / aCarre
+    KLamb = 11565915.812935
+    nLamb = 0.7716421928
+    
+    eLamb  = math.Sqrt(eCarre)
+    eSur2  = eLamb / 2
+    
+    #conversion to radians
+    lat = (math.PI / 180) * latBel
+    lng = (math.PI / 180) * lngBel
+    
+    eSinLatitude = eLamb * math.Sin(lat)
+    TanZDemi = (math.Tan((math.PI / 4) - (lat / 2))) *  (((1 + (eSinLatitude)) / (1 - (eSinLatitude))) ^ (eSur2))
+    
+    RLamb= KLamb * ((TanZDemi) ^ nLamb)
+    
+    Teta  = nLamb * (lng - LongRef)
+        
+    x = 150000 + 0.01256 + RLamb * math.Sin(Teta - 0.000142043)
+    y = 5400000 + 88.4378 - RLamb * math.Cos(Teta - 0.000142043)
+    return x,y
+
+def lambert72_to_spherical_coordinates(x :float ,y:float) -> tuple: 
+    """"
+    Belgian Lambert 1972---> Spherical coordinates
+    Input parameters : X, Y = Belgian coordinates in meters
+    Output : latitude and longitude in Belgium Datum!
+    source: http://zoologie.umons.ac.be/tc/algorithms.aspx
+    """
+    LongRef = 0.076042943  #      '=4°21'24"983
+    nLamb  = 0.7716421928 #
+    aCarre = 6378388 ^ 2 #
+    bLamb = 6378388 * (1 - (1 / 297)) #
+    eCarre = (aCarre - bLamb ^ 2) / aCarre #
+    KLamb = 11565915.812935 #
+    
+    eLamb = math.Sqrt(eCarre)
+    eSur2 = eLamb / 2
+    
+    Tan1  = (x - 150000.01256) / (5400088.4378 - y)
+    Lambda = LongRef + (1 / nLamb) * (0.000142043 + math.Atan(Tan1))
+    RLamb = math.Sqrt((x - 150000.01256) ^ 2 + (5400088.4378 - y) ^ 2)
+    
+    TanZDemi = (RLamb / KLamb) ^ (1 / nLamb)
+    Lati1 = 2 * math.Atan(TanZDemi)
+    
+    eSin=0.0
+    Mult1=0.0
+    Mult2=0.0
+    Mult=0.0
+    LatiN=0.0
+    Diff=1     
+
+    while math.Abs(Diff) > 0.0000000277777:
+        eSin = eLamb * math.Sin(Lati1)
+        Mult1 = 1 - eSin
+        Mult2 = 1 + eSin
+        Mult = (Mult1 / Mult2) ^ (eLamb / 2)
+        LatiN = (math.PI / 2) - (2 * (math.Atan(TanZDemi * Mult)))
+        Diff = LatiN - Lati1
+        Lati1 = LatiN
+    
+    latBel = (LatiN * 180) / math.PI
+    lngBel = (Lambda * 180) / math.PI
+    return latBel,lngBel
+
+
+#### FORMAT CONVERTERS ####
+
 def cartesian_bounds_to_cartesian_transform(cartesianBounds:np.array) ->np.array:
     if len(cartesianBounds) !=6:
         return None
@@ -117,56 +246,132 @@ def e57_to_pcd(e57Path, e57Index) ->o3d.geometry.PointCloud:
         print("Conversion from e57 to o3d.geometry.PointCloud failed!")
         return None    
 
-def e57_get_colors(raw_data):
-    """
-    Extract color of intensity information from e57 raw data (3D data) and output Open3D o3d.utility.Vector3dVector(colors) 
-    """   
+def box_to_mesh(box:o3d.geometry) ->o3d.geometry.TriangleMesh:
+    if type(box) is o3d.geometry.OrientedBoundingBox or type(box) is o3d.geometry.AxisAlignedBoundingBox:
+        mesh=o3d.geometry.TriangleMesh()
+        mesh.vertices=box.get_box_points()
+        #triangles rotate counterclockwise
+        mesh.triangles= o3d.cpu.pybind.utility.Vector3iVector(np.array([[0,2,1],
+                            [0,1,3],
+                            [0,3,2],
+                            [1,6,3],
+                            [1,7,6],
+                            [1,2,7],
+                            [2,3,5],
+                            [2,5,4],
+                            [2,4,7],
+                            [3,4,5],
+                            [3,6,4],
+                            [4,6,7]])) 
+        return mesh                  
+    else:
+        print('Incorrect geometry input. Only input o3d.geometry.AxisAlignedBoundingBox or o3d.geometry.OrientedBoundingBox or type(geometry)  ')
+        return None 
+
+def ifc_to_mesh(ifcElement:ifcopenshell.entity_instance)-> o3d.geometry.TriangleMesh: 
+    """Convert an ifcOpenShell geometry to an Open3D TriangleMesh
+
+    Args:
+        ifcElement (ifcopenshell.entity_instance): IfcOpenShell Element parsed from and .ifc file. See BIMNode for more documentation.
+
+    Returns:
+        o3d.geometry.TriangleMesh: Open3D Mesh Geometry of the ifcElment boundary surface
+    """    
     try:
-        colorRed=raw_data.get('colorRed')
-        colorGreen=raw_data.get('colorGreen')
-        colorBlue=raw_data.get('colorBlue')
-        intensity=raw_data.get('intensity')
+        if ifcElement.get_info().get("Representation"): 
+            # Set geometry settings and global coordinates as true
+            settings = geom.settings() 
+            settings.set(settings.USE_WORLD_COORDS, True) 
+        
+            # Extract vertices/faces of the IFC geometry
+            shape = geom.create_shape(settings, ifcElement) 
+            ifcVertices = shape.geometry.verts 
+            ifcFaces = shape.geometry.faces 
 
-        if colorRed is not None and colorGreen is not None and colorBlue is not None:
-            if np.amax(colorRed)<=1:
-                colors=np.c_[colorRed , colorGreen,colorBlue ]  
-            elif np.amax(colorRed) <=255:
-                colors=np.c_[colorRed/255 , colorGreen/255,colorBlue/255 ]  
-            else:
-                colorRed=(colorRed - np.min(colorRed))/np.ptp(colorRed)
-                colorGreen=(colorGreen - np.min(colorGreen))/np.ptp(colorGreen)
-                colorBlue=(colorBlue - np.min(colorBlue))/np.ptp(colorBlue)
-                colors=np.c_[colorRed , colorGreen,colorBlue ]  
-            return o3d.utility.Vector3dVector(colors) 
+            #Group the vertices and faces in a way they can be read by Open3D
+            vertices = [[ifcVertices[i], ifcVertices[i + 1], ifcVertices[i + 2]] for i in range(0, len(ifcVertices), 3)]
+            faces = [[ifcFaces[i], ifcFaces[i + 1], ifcFaces[i + 2]] for i in range(0, len(ifcFaces), 3)]
 
-        elif intensity is not None:
-            if np.amax(intensity) <=1:
-                colors=np.c_[intensity , intensity,intensity ]  
-            else:
-                intensity=(intensity - np.min(intensity))/np.ptp(intensity)
-                colors=np.c_[intensity , intensity,intensity ]  
-            return o3d.utility.Vector3dVector(colors) 
+            #Convert grouped vertices/faces to Open3D objects 
+            o3dVertices = o3d.utility.Vector3dVector(np.asarray(vertices))
+            o3dTriangles = o3d.utility.Vector3iVector(np.asarray(faces))
+
+            # Create the Open3D mesh object
+            mesh=o3d.geometry.TriangleMesh(o3dVertices,o3dTriangles)
+            if len(mesh.triangles)>1:
+                return mesh
+            else: 
+                return None 
     except:
+        print('ifcopenshell.entity_instance error')
         return None
-    return None
+
+def cartesian_bounds_to_open3d_axis_aligned_bounding_box(cartesianBounds:np.array) ->o3d.geometry.OrientedBoundingBox:
+    """Convert cartesianBounds to Open3D Geometry
+
+    Args:
+        cartesianBounds (np.array): [xMin,xMax,yMin,yMax,zMin,zMax]
+
+    Returns:
+        o3d.geometry.OrientedBoundingBox: Open3D Geometry
+    """
+    points=get_bounding_points(cartesianBounds)
+    open3dOrientedBoundingBox=o3d.geometry.OrientedBoundingBox.create_from_points(points)
+    return open3dOrientedBoundingBox
+
+def rotation_and_translation_to_transformation_matrix(rotation: np.array, translation: np.array)->np.array:
+    """
+    rotation [3,3]
+    translation [3,1]
+    """
+    r=np.asarray(rotation)
+    if r.size==9 and len(translation) ==3:    
+        return np.array([[ r[0,0],r[0,1],r[0,2], translation[0]],
+                        [ r[1,0],r[1,1],r[1,2], translation[1]],
+                        [ r[2,0],r[2,1],r[2,2], translation[2]],
+                        [ 0,0,0,1]])
+    else:
+        return None
+
+def mesh_to_pcd(mesh:o3d.geometry.TriangleMesh,voxelSize : float = 0.1) -> o3d.geometry.PointCloud:
+    """Sample a point cloud on a triangle mesh (Open3D)
+
+    Args:
+        mesh (o3d.geometry.TriangleMesh) : source geometry
+        voxel_size (float) : spatial resolution of the point cloud e.g. 0.1m
+
+    Returns:
+        An o3d.geometry.PointCloud() 
+    """
+    pcd = o3d.geometry.PointCloud()
+    for submesh in mesh:
+        k = round(submesh.get_surface_area() * 1000)
+        submeshpcd = submesh.sample_points_uniformly(number_of_points = k, use_triangle_normal=True)
+        pcd.__iadd__(submeshpcd)
+    pcd = pcd.voxel_down_sample(voxelSize)    
+    return pcd
+
+def oriented_bounds_to_open3d_oriented_bounding_box(orientedBounds : np.array) -> o3d.geometry.OrientedBoundingBox:
+    """Convert 8 bounding points to Open3D Geometry
+
+    Args:
+        orientedBounds (np.array): [8x3] bounding points
+
+    Returns:
+        o3d.geometry.OrientedBoundingBox: Open3D Geometry
+    """
+    boxPointsVector = o3d.utility.Vector3dVector(orientedBounds)
+    open3dOrientedBoundingBox = o3d.geometry.OrientedBoundingBox.create_from_points(boxPointsVector)
+    return open3dOrientedBoundingBox
 
 
-def e57_fix_rotation_order(rotation_matrix:np.array) -> np.array:
-    """
-    thix switches the rotation from clockwise to counter-clockwise
-    https://robotics.stackexchange.com/questions/10702/rotation-matrix-sign-convention-confusion
-    Currently only changes the signs of elements [0,1,5,8]
-    """
-    r=rotation_matrix
-    return [[-r[0,0],-r[0,1],r[0,2]],
-            [r[1,0],r[1,1],-r[1,2]],
-            [r[2,0],r[2,1],-r[2,2]]]
+#### GEOMETRY GENERATION ####
 
 def generate_virtual_images(geometries: List[o3d.geometry.Geometry],cartesianTransforms: List[np.array],width:int=640,height:int=480,f:float=400)-> o3d.geometry.Image:
     """Generate a set of Open3D Images from cartesianTransforms and geometries. 
     The same intrinsic camera parameters are used for all cartesianTransforms that are passed to the function
 
-    .. image:: ../../pics/rendering3.PNG
+    .. image:: ../pics/rendering3.PNG
 
     Args:
         geometries (List[o3d.geometry]):o3d.geometry.PointCloud or o3d.geometry.TriangleMesh
@@ -247,10 +452,56 @@ def generate_virtual_image(geometries: List[o3d.geometry.Geometry],pinholeCamera
         return None
 
 
+#### GEOMETRY MANIPULATION ####
+
+def e57_get_colors(raw_data):
+    """
+    Extract color of intensity information from e57 raw data (3D data) and output Open3D o3d.utility.Vector3dVector(colors) 
+    """   
+    try:
+        colorRed=raw_data.get('colorRed')
+        colorGreen=raw_data.get('colorGreen')
+        colorBlue=raw_data.get('colorBlue')
+        intensity=raw_data.get('intensity')
+
+        if colorRed is not None and colorGreen is not None and colorBlue is not None:
+            if np.amax(colorRed)<=1:
+                colors=np.c_[colorRed , colorGreen,colorBlue ]  
+            elif np.amax(colorRed) <=255:
+                colors=np.c_[colorRed/255 , colorGreen/255,colorBlue/255 ]  
+            else:
+                colorRed=(colorRed - np.min(colorRed))/np.ptp(colorRed)
+                colorGreen=(colorGreen - np.min(colorGreen))/np.ptp(colorGreen)
+                colorBlue=(colorBlue - np.min(colorBlue))/np.ptp(colorBlue)
+                colors=np.c_[colorRed , colorGreen,colorBlue ]  
+            return o3d.utility.Vector3dVector(colors) 
+
+        elif intensity is not None:
+            if np.amax(intensity) <=1:
+                colors=np.c_[intensity , intensity,intensity ]  
+            else:
+                intensity=(intensity - np.min(intensity))/np.ptp(intensity)
+                colors=np.c_[intensity , intensity,intensity ]  
+            return o3d.utility.Vector3dVector(colors) 
+    except:
+        return None
+    return None
+
+def e57_fix_rotation_order(rotation_matrix:np.array) -> np.array:
+    """
+    thix switches the rotation from clockwise to counter-clockwise
+    https://robotics.stackexchange.com/questions/10702/rotation-matrix-sign-convention-confusion
+    Currently only changes the signs of elements [0,1,5,8]
+    """
+    r=rotation_matrix
+    return [[-r[0,0],-r[0,1],r[0,2]],
+            [r[1,0],r[1,1],-r[1,2]],
+            [r[2,0],r[2,1],-r[2,2]]]
+
 def crop_geometry(geometry:o3d.geometry, box:o3d.geometry.OrientedBoundingBox, subdivide:int = 0) ->o3d.geometry.TriangleMesh:
     """"Crop portion of a mesh/pointcloud that lies within an OrientedBoundingBox 
 
-    .. image:: ../../pics/selection_BB_mesh2.PNG
+    .. image:: ../pics/selection_BB_mesh2.PNG
 
     Args:
         geometry (o3d.geometry.TriangleMesh or o3d.geometry.PointCloud)
@@ -309,17 +560,6 @@ def expand_box (box: o3d.geometry.OrientedBoundingBox, u=5.0,v=5.0,w=5.0) -> o3d
     extent = box.extent + [u, w, v] 
     return o3d.geometry.OrientedBoundingBox(center,orientation,extent) 
 
-
-# def crop_mesh2(mesh:o3d.geometry.TriangleMesh, geometry:o3d.geometry) ->o3d.geometry.TriangleMesh:
-#     """"
-#     NOT IMPLEMENTED
-#     """
-#     # test which indices are within box
-#     hull=geometry.compute_convex_hull()
-
-#     box.get_box_points()
-
-
 def join_geometries(geometries):
     if type(geometries) is o3d.geometry.TriangleMesh:
         joined=o3d.geometry.TriangleMesh()
@@ -369,27 +609,8 @@ def pointcloud_filter_by_distance(source: o3d.geometry.PointCloud, cutters, thre
     else:
         return None 
 
-def box_to_mesh(box:o3d.geometry) ->o3d.geometry.TriangleMesh:
-    if type(box) is o3d.geometry.OrientedBoundingBox or type(box) is o3d.geometry.AxisAlignedBoundingBox:
-        mesh=o3d.geometry.TriangleMesh()
-        mesh.vertices=box.get_box_points()
-        #triangles rotate counterclockwise
-        mesh.triangles= o3d.cpu.pybind.utility.Vector3iVector(np.array([[0,2,1],
-                            [0,1,3],
-                            [0,3,2],
-                            [1,6,3],
-                            [1,7,6],
-                            [1,2,7],
-                            [2,3,5],
-                            [2,5,4],
-                            [2,4,7],
-                            [3,4,5],
-                            [3,6,4],
-                            [4,6,7]])) 
-        return mesh                  
-    else:
-        print('Incorrect geometry input. Only input o3d.geometry.AxisAlignedBoundingBox or o3d.geometry.OrientedBoundingBox or type(geometry)  ')
-        return None 
+
+#### GEOMETRY PROPERTIES
 
 def get_box_inliers(sourceBox:o3d.geometry.OrientedBoundingBox, testBoxes: List[o3d.geometry.OrientedBoundingBox])->List[int]:
     """"
@@ -458,71 +679,6 @@ def get_box_intersections(sourceBox:o3d.geometry.OrientedBoundingBox, testBoxes:
     list = [ i for i in range(len(array)) if array[i]]
     return list
 
-
-def rotation_and_translation_to_transformation_matrix(rotation: np.array, translation: np.array)->np.array:
-    """
-    rotation [3,3]
-    translation [3,1]
-    """
-    r=np.asarray(rotation)
-    if r.size==9 and len(translation) ==3:    
-        return np.array([[ r[0,0],r[0,1],r[0,2], translation[0]],
-                        [ r[1,0],r[1,1],r[1,2], translation[1]],
-                        [ r[2,0],r[2,1],r[2,2], translation[2]],
-                        [ 0,0,0,1]])
-    else:
-        return None
-
-
-def ifc_to_mesh(ifcElement:ifcopenshell.entity_instance)-> o3d.geometry.TriangleMesh: 
-    """Convert an ifcOpenShell geometry to an Open3D TriangleMesh
-
-    Args:
-        ifcElement (ifcopenshell.entity_instance): IfcOpenShell Element parsed from and .ifc file. See BIMNode for more documentation.
-
-    Returns:
-        o3d.geometry.TriangleMesh: Open3D Mesh Geometry of the ifcElment boundary surface
-    """    
-    try:
-        if ifcElement.get_info().get("Representation"): 
-            # Set geometry settings and global coordinates as true
-            settings = geom.settings() 
-            settings.set(settings.USE_WORLD_COORDS, True) 
-        
-            # Extract vertices/faces of the IFC geometry
-            shape = geom.create_shape(settings, ifcElement) 
-            ifcVertices = shape.geometry.verts 
-            ifcFaces = shape.geometry.faces 
-
-            #Group the vertices and faces in a way they can be read by Open3D
-            vertices = [[ifcVertices[i], ifcVertices[i + 1], ifcVertices[i + 2]] for i in range(0, len(ifcVertices), 3)]
-            faces = [[ifcFaces[i], ifcFaces[i + 1], ifcFaces[i + 2]] for i in range(0, len(ifcFaces), 3)]
-
-            #Convert grouped vertices/faces to Open3D objects 
-            o3dVertices = o3d.utility.Vector3dVector(np.asarray(vertices))
-            o3dTriangles = o3d.utility.Vector3iVector(np.asarray(faces))
-
-            # Create the Open3D mesh object
-            mesh=o3d.geometry.TriangleMesh(o3dVertices,o3dTriangles)
-            if len(mesh.triangles)>1:
-                return mesh
-            else: 
-                return None 
-    except:
-        print('ifcopenshell.entity_instance error')
-        return None
-  
-
-
-def save_view_point(geometry, filename):
-    vis = o3d.visualization.Visualizer()
-    vis.create_window()
-    vis.add_geometry(geometry)
-    vis.run()  # user changes the view and press "q" to terminate
-    param = vis.get_view_control().convert_to_pinhole_camera_parameters()
-    o3d.io.write_pinhole_camera_parameters(filename, param)
-    vis.destroy_window()
-
 def get_center(mesh:o3d.geometry.TriangleMesh,triangle:np.array) -> np.array:
     """
     Compute center of an Open3D mesh face
@@ -548,7 +704,6 @@ def get_triangle_centers(mesh:o3d.geometry.TriangleMesh) -> np.array:
         centers[idx]=np.mean(points,axis=0)
     return centers
 
-
 def get_cartesian_bounds(geometry : o3d.geometry) ->np.array:
     """Get cartesian bounds from Open3D geometry
 
@@ -574,20 +729,6 @@ def get_oriented_bounds(geometry : o3d.geometry.OrientedBoundingBox ) -> np.arra
     orientedBoundingBox = geometry.get_oriented_bounding_box()
     return np.asarray(orientedBoundingBox.get_box_points())
 
-
-def cartesian_bounds_to_open3d_axis_aligned_bounding_box(cartesianBounds:np.array) ->o3d.geometry.OrientedBoundingBox:
-    """Convert cartesianBounds to Open3D Geometry
-
-    Args:
-        cartesianBounds (np.array): [xMin,xMax,yMin,yMax,zMin,zMax]
-
-    Returns:
-        o3d.geometry.OrientedBoundingBox: Open3D Geometry
-    """
-    points=get_bounding_points(cartesianBounds)
-    open3dOrientedBoundingBox=o3d.geometry.OrientedBoundingBox.create_from_points(points)
-    return open3dOrientedBoundingBox
-
 def get_center_of_oriented_bounds(orientedBounds: np.array) -> np.array:
     boxPointsVector = o3d.utility.Vector3dVector(orientedBounds)
     open3dOrientedBoundingBox = o3d.geometry.OrientedBoundingBox.create_from_points(boxPointsVector)
@@ -648,49 +789,8 @@ def get_rotation(cartesianTransform:np.array) -> np.array:
     else:
         return None
 
-def lambert72_to_spherical_coordinates(x :float ,y:float) -> tuple: 
-    """"
-    Belgian Lambert 1972---> Spherical coordinates
-    Input parameters : X, Y = Belgian coordinates in meters
-    Output : latitude and longitude in Belgium Datum!
-    source: http://zoologie.umons.ac.be/tc/algorithms.aspx
-    """
-    LongRef = 0.076042943  #      '=4°21'24"983
-    nLamb  = 0.7716421928 #
-    aCarre = 6378388 ^ 2 #
-    bLamb = 6378388 * (1 - (1 / 297)) #
-    eCarre = (aCarre - bLamb ^ 2) / aCarre #
-    KLamb = 11565915.812935 #
-    
-    eLamb = math.Sqrt(eCarre)
-    eSur2 = eLamb / 2
-    
-    Tan1  = (x - 150000.01256) / (5400088.4378 - y)
-    Lambda = LongRef + (1 / nLamb) * (0.000142043 + math.Atan(Tan1))
-    RLamb = math.Sqrt((x - 150000.01256) ^ 2 + (5400088.4378 - y) ^ 2)
-    
-    TanZDemi = (RLamb / KLamb) ^ (1 / nLamb)
-    Lati1 = 2 * math.Atan(TanZDemi)
-    
-    eSin=0.0
-    Mult1=0.0
-    Mult2=0.0
-    Mult=0.0
-    LatiN=0.0
-    Diff=1     
 
-    while math.Abs(Diff) > 0.0000000277777:
-        eSin = eLamb * math.Sin(Lati1)
-        Mult1 = 1 - eSin
-        Mult2 = 1 + eSin
-        Mult = (Mult1 / Mult2) ^ (eLamb / 2)
-        LatiN = (math.PI / 2) - (2 * (math.Atan(TanZDemi * Mult)))
-        Diff = LatiN - Lati1
-        Lati1 = LatiN
-    
-    latBel = (LatiN * 180) / math.PI
-    lngBel = (Lambda * 180) / math.PI
-    return latBel,lngBel
+#### GEOMETRY TESTING ####
 
 def mesh_collisions_open3d(sourceMesh: o3d.geometry.TriangleMesh, meshes: List[o3d.geometry.TriangleMesh], t_d:float=0.5) -> List[int]:
     """Return the index of the geometries that collide the sourceMesh
@@ -788,119 +888,90 @@ def mesh_intersection_raycasting(source:o3d.geometry.TriangleMesh, cutter: o3d.g
         outputmesh.remove_unreferenced_vertices()
     return outputmesh
 
-def mesh_to_pcd(mesh:o3d.geometry.TriangleMesh,voxelSize : float = 0.1) -> o3d.geometry.PointCloud:
-    """Sample a point cloud on a triangle mesh (Open3D)
+def voxel_traversal(pcd, origin, direction, voxelSize, maxRange = 100): 
+    """Cast a ray in a voxel array created from the pcd"""
 
-    Args:
-        mesh (o3d.geometry.TriangleMesh) : source geometry
-        voxel_size (float) : spatial resolution of the point cloud e.g. 0.1m
+    pcd_Voxel = o3d.geometry.VoxelGrid.create_from_point_cloud(pcd,voxel_size=voxelSize)
+    #the starting index of the voxels
+    voxel_origin = pcd.get_voxel(origin)
+    iX = math.floor(origin[0] * voxelSize) / voxelSize
+    iY = math.floor(origin[1] * voxelSize) / voxelSize
+    iZ = math.floor(origin[2] * voxelSize) / voxelSize
 
-    Returns:
-        An o3d.geometry.PointCloud() 
-    """
-    pcd = o3d.geometry.PointCloud()
-    for submesh in mesh:
-        k = round(submesh.get_surface_area() * 1000)
-        submeshpcd = submesh.sample_points_uniformly(number_of_points = k, use_triangle_normal=True)
-        pcd.__iadd__(submeshpcd)
-    pcd = pcd.voxel_down_sample(voxelSize)    
-    return pcd
+    stepX = np.sign(direction[0])
+    stepY = np.sign(direction[1])
+    stepZ = np.sign(direction[2])
 
-def oriented_bounds_to_open3d_oriented_bounding_box(orientedBounds : np.array) -> o3d.geometry.OrientedBoundingBox:
-    """Convert 8 bounding points to Open3D Geometry
+    tDeltaX = 1/direction[0] * voxelSize
+    tDeltaY = 1/direction[1] * voxelSize
+    tDeltaZ = 1/direction[2] * voxelSize
 
-    Args:
-        orientedBounds (np.array): [8x3] bounding points
+    tMaxX = origin[0]
+    tMaxY = origin[1]
+    tMaxZ = origin[2]
 
-    Returns:
-        o3d.geometry.OrientedBoundingBox: Open3D Geometry
-    """
-    boxPointsVector = o3d.utility.Vector3dVector(orientedBounds)
-    open3dOrientedBoundingBox = o3d.geometry.OrientedBoundingBox.create_from_points(boxPointsVector)
-    return open3dOrientedBoundingBox
+    for i in range(0,maxRange):
+        # check if the current point is in a occupied voxel
+        if(pcd_Voxel.check_if_included(o3d.utility.Vector3dVector([[iX * voxelSize, iY * voxelSize, iY * voxelSize]]))[0]):
+            distance = np.linalg.norm(np.array([iX * voxelSize, iY * voxelSize, iY * voxelSize]) - origin)
+            return True, distance
 
-def spherical_coordinates_to_lambert72(latBel :float,lngBel:float) -> tuple:
-    """
-    Conversion from spherical coordinates to Lambert 72
-    Input parameters : lat, lng (spherical coordinates)
-    Spherical coordinates are in decimal degrees converted to Belgium datum!
-    source: http://zoologie.umons.ac.be/tc/algorithms.aspx
-    """
- 
-    LongRef  = 0.076042943  #      '=4°21'24"983
-    bLamb  = 6378388 * (1 - (1 / 297))
-    aCarre  = 6378388 ^ 2
-    eCarre = (aCarre - bLamb ^ 2) / aCarre
-    KLamb = 11565915.812935
-    nLamb = 0.7716421928
-    
-    eLamb  = math.Sqrt(eCarre)
-    eSur2  = eLamb / 2
-    
-    #conversion to radians
-    lat = (math.PI / 180) * latBel
-    lng = (math.PI / 180) * lngBel
-    
-    eSinLatitude = eLamb * math.Sin(lat)
-    TanZDemi = (math.Tan((math.PI / 4) - (lat / 2))) *  (((1 + (eSinLatitude)) / (1 - (eSinLatitude))) ^ (eSur2))
-    
-    RLamb= KLamb * ((TanZDemi) ^ nLamb)
-    
-    Teta  = nLamb * (lng - LongRef)
-        
-    x = 150000 + 0.01256 + RLamb * math.Sin(Teta - 0.000142043)
-    y = 5400000 + 88.4378 - RLamb * math.Cos(Teta - 0.000142043)
-    return x,y
+        if(tMaxX < tMaxY):
+            if(tMaxX < tMaxZ):
+                #traverse in the X direction
+                tMaxX += tDeltaX
+                iX += stepX
+            else:
+                #traverse in the Z direction
+                tMaxZ += tDeltaZ
+                iZ += stepZ
+        else:
+            if(tMaxY < tMaxZ):
+                #traverse in the Y direction
+                tMaxY += tDeltaY
+                iY += stepY
+            else:
+                #traverse in the Z direction
+                tMaxZ += tDeltaZ
+                iZ += stepZ
 
-def visualize_geometry(geometry)->None:
+    return False, math.inf
+
+
+#### OPEN3D VISUALISATION ####
+
+def create_3d_camera(pos = [0,0,0], rotation  = np.eye(3), scale = 1.0):
+    "Returns a geometry lineset object that represents a camera in 3D space"
+
+    box = o3d.geometry.TriangleMesh.create_box(1.6,0.9, 0.1)
+    box.translate((-0.8, -0.45, -0.05))
+    box.scale(scale, center=(0, 0, 0))
+    box.rotate(rotation)
+    box.translate(pos)
+    return box
+
+def show_geometries(geometries, color = False):
+    "displays the array of meshes in a 3D view"
+
+    viewer = o3d.visualization.Visualizer()
+    viewer.create_window()
+    frame = o3d.geometry.TriangleMesh.create_coordinate_frame()
+    viewer.add_geometry(frame)
+    for i, geometry in enumerate(geometries):
+        if color:
+            geometry.paint_uniform_color(matplotlib.colors.hsv_to_rgb([float(i)/len(geometries),0.8,0.8]))
+        viewer.add_geometry(geometry)
+    opt = viewer.get_render_option()
+    opt.background_color = np.asarray([1,1,1])
+    opt.light_on = True
+    viewer.run()
+    viewer.destroy_window()
+
+def save_view_point(geometry, filename):
     vis = o3d.visualization.Visualizer()
     vis.create_window()
     vis.add_geometry(geometry)
-    vis.run()
+    vis.run()  # user changes the view and press "q" to terminate
+    param = vis.get_view_control().convert_to_pinhole_camera_parameters()
+    o3d.io.write_pinhole_camera_parameters(filename, param)
     vis.destroy_window()
-
-
-def wgs84_to_belgian_datum(LatWGS84: float,LngWGS84:float) -> tuple:
-    """
-    Input parameters : Lat, Lng : latitude / longitude in decimal degrees and in WGS84 datum
-    Output parameters : LatBel, LngBel : latitude / longitude in decimal degrees and in Belgian datum
-    source: http://zoologie.umons.ac.be/tc/algorithms.aspx
-    """
-    Haut = 0    
-    
-    #conversion to radians
-    Lat = (math.PI / 180) * LatWGS84
-    Lng = (math.PI / 180) * LngWGS84
-    
-    SinLat = math.Sin(Lat)
-    SinLng = math.Sin(Lng)
-    CoSinLat = math.Cos(Lat)
-    CoSinLng = math.Cos(Lng)
-    
-    dx = 125.8
-    dy = -79.9
-    dz = 100.5
-    da = 251.0
-    df = 0.000014192702
-    
-    LWf = 1 / 297
-    LWa = 6378388
-    LWb = (1 - LWf) * LWa
-    LWe2 = (2 * LWf) - (LWf * LWf)
-    Adb = 1 / (1 - LWf)
-    
-    Rn = LWa / math.Sqrt(1 - LWe2 * SinLat * SinLat)
-    Rm = LWa * (1 - LWe2) / (1 - LWe2 * Lat * Lat) ^ 1.5
-    
-    DLat = -dx * SinLat * CoSinLng - dy * SinLat * SinLng + dz * CoSinLat
-    DLat = DLat + da * (Rn * LWe2 * SinLat * CoSinLat) / LWa
-    DLat = DLat + df * (Rm * Adb + Rn / Adb) * SinLat * CoSinLat
-    DLat = DLat / (Rm + Haut)
-    
-    DLng = (-dx * SinLng + dy * CoSinLng) / ((Rn + Haut) * CoSinLat)
-    Dh = dx * CoSinLat * CoSinLng + dy * CoSinLat * SinLng + dz * SinLat
-    Dh = Dh - da * LWa / Rn + df * Rn * Lat * Lat / Adb
-    
-    LatBel = ((Lat + DLat) * 180) / math.PI
-    LngBel = ((Lng + DLng) * 180) / math.PI
-    return  LatBel,LngBel
