@@ -2,6 +2,7 @@
 ImageNode - a Python Class to govern the data and metadata of image data (JPG,PNG,XML,XMP)
 """
 #IMPORT PACKAGES
+from distutils import extension
 import xml.etree.ElementTree as ET
 import cv2
 import PIL
@@ -81,7 +82,8 @@ class ImageNode(Node):
         # where do we store image classifications     
         
         if path is not None:
-            if path.endswith(ut.IMG_EXTENSION): #data + metadata
+            extension=ut.get_extension(path)
+            if extension in ut.IMG_EXTENSION:
                 self.name=ut.get_filename(path)
                 self.timestamp=ut.get_timestamp(path)                
                 self.get_metadata_from_exif_data()
@@ -212,8 +214,14 @@ class ImageNode(Node):
                 gps_info = exifData["GPSInfo"]
                 if gps_info is not None:
                     # self.GlobalPose=GlobalPose # (structure) SphericalTranslation(lat,long,alt), Quaternion(qw,qx,qy,qz)
-                    self.geospatialTransform=[  ut.get_if_exist(gps_info, "GPSLatitude"), 
-                                                ut.get_if_exist(gps_info, "GPSLongitude"),
+                    latitude=ut.get_if_exist(gps_info, "GPSLatitude")
+                    latReference=ut.get_if_exist(gps_info, "GPSLatitudeRef")
+                    newLatitude=ut.filter_exif_gps_data(latitude,latReference)
+                    longitude= ut.get_if_exist(gps_info, "GPSLongitude")
+                    longReference=ut.get_if_exist(gps_info, "GPSLongitudeRef")
+                    newLongitude=ut.filter_exif_gps_data(longitude,longReference)
+                    self.geospatialTransform=[  newLatitude, 
+                                                newLongitude,
                                                 ut.get_if_exist(gps_info, "GPSAltitude")]
                     self.coordinateSystem='geospatial-wgs84'
             return True
