@@ -116,7 +116,7 @@ def e57header_to_nodes(e57Path:str, **kwargs) -> List[PointCloudNode]:
         print('e57header error')
         return None
 
-def ifc_to_nodes(ifcPath:str, classes:str='ifcElement',getGeometry=True,**kwargs)-> List[BIMNode]:
+def ifc_to_nodes(ifcPath:str, classes:str='ifcElement',getResource : bool=True,**kwargs)-> List[BIMNode]:
     """
     Parse ifc file to a list of BIMNodes, one for each ifcElement
 
@@ -132,13 +132,12 @@ def ifc_to_nodes(ifcPath:str, classes:str='ifcElement',getGeometry=True,**kwargs
         selector = Selector()
         for ifcElement in selector.parse(ifc, classes): 
             try:   
-                node=BIMNode(ifcElement=ifcElement,ifcPath=ifcPath,getGeometry=getGeometry, **kwargs)
+                node=BIMNode(ifcElement=ifcElement,ifcPath=ifcPath,getResource=getResource, **kwargs)
                 if getattr(node,'mesh',None) is not None and len(node.mesh.triangles) !=0:
                     node.get_metadata_from_mesh()
             except:
-                print('Node creation error')
+                print('Node '+ ifcElement.Name + ' creation error')
                 continue
-            # if node.set_metadata_from_ifc_element(ifcElement):
             nodelist.append(node)
         return nodelist
     except:
@@ -359,17 +358,24 @@ def select_nodes_with_intersecting_meshes(node:Node,nodelist:List[Node]) -> List
 
 #### GRAPH CREATION #####
 
-def nodes_to_graph(nodelist : List[Node]) -> Graph:
+def nodes_to_graph(nodelist : List[Node], graphPath:str =None) -> Graph:
     """ Convert list of nodes to a graph"""
     g=Graph()
     g=ut.bind_ontologies(g)
     for idx,node in enumerate(nodelist):
         try:
-            node.to_graph()
-            g+=node.graph
+            node.to_graph(graphPath)
+            g+=node.graph            
+           
         except:
             print('Node '+str(idx)+' could not be serialized.')
             continue
+    if(graphPath):
+        try: 
+            open(graphPath, 'w')
+            g.serialize(destination=graphPath, format='ttl')
+        except:
+            print('graphPath invalid.')        
     return g  
 
 #### OBSOLETE #####
@@ -384,14 +390,12 @@ def graph_to_nodes(graph : Graph,**kwargs) -> List[Node]:
     Returns:
         A list of pointcloudnodes, imagenodes, meshnodes, bimnodes, orthonodes with metadata 
     """
-    warn("This function is depricated use a SessionNode instead")
     nodelist=[]
     for subject in graph.subjects(RDF.type): #iterate over subjects
-        #create correct nodeType
+        
+        # graph = Graph()
+        # graph += graph.triples((subject, None, None))
         node=subject_to_node_type(graph,subject,**kwargs)
-        node.name = str(subject).replace('http://','') 
-        node.graph = Graph()
-        node.graph += graph.triples((subject, None, None))
 
         for predicate,object in graph.predicate_objects(subject):# iterate over all predicates of a subject
             attr=ut.predicate_to_attribute(predicate)
@@ -434,7 +438,9 @@ def graph_to_nodes(graph : Graph,**kwargs) -> List[Node]:
     return nodelist
 
 def subject_to_node_type(graph: Graph , subject:URIRef, **kwargs)-> Node:
-    warn("This function is depricated use a SessionNode instead")
+    # warn("This function is depricated use a SessionNode instead")
+
+
     nodeType = ut.literal_to_string(graph.value(subject=subject,predicate=RDF.type))
     g = Graph()
     g += graph.triples((subject, None, None))
