@@ -1,10 +1,11 @@
 """
-Node - a Python Class to govern the data and metadata of remote sensing data (pcd, images, meshes, orthomozaik)
+Node - an abstract Python Class to govern the data and metadata of remote sensing data (pcd, images, meshes, orthomozaik)
 """
 
 #IMPORT PACKAGES
 import os
 import re
+import uuid
 
 from rdflib import Graph, URIRef, Literal
 from rdflib.namespace import RDF
@@ -22,7 +23,7 @@ class Node:
 
     def __init__(self,  graph: Graph = None,
                         graphPath: str = None,
-                        subject: URIRef = None,
+                        subject: str = None,
                         **kwargs):
         """Creates a new node
 
@@ -33,7 +34,7 @@ class Node:
 
         Returns:
             Args(None): Create an empty graph with an unique subject guid
-            Args(graph-path): parse the graph and choose the first subject in the graph
+            Args(graphPath): parse the graph and choose the first subject in the graph
             Args(subject): create a new graph with the given subject
             Args(subject & graph-path): parse the graph with the given subject
         """
@@ -47,11 +48,10 @@ class Node:
         elif (graphPath):   # if we only get a path, load the path and parse the graph
             self.get_metadata_from_graph_path()
         else:               # No graph is given, create a new one
-            self.to_graph()
+            self.get_subject()  
+            # self.to_graph()
 
-        # Step 2: Assign all the optional values to the new node
-
-        self.__dict__.update(kwargs) 
+        self.__dict__.update(kwargs)  
 
     def get_metadata_from_graph(self, graph: Graph = None, subject: URIRef = None):
         """Convert a graph to a node object with a single subject, if there are more subjects, only the first is parsed.
@@ -62,7 +62,7 @@ class Node:
             subject (RDFlib.URIRef): The subject to parse the graph for, leave blank to use the instance subject. Defaults to None.
         
         """
-        # Get the subjects and the graph
+        # Get the subject from the graph. if no subject is presented, the first subject is retained
         self.get_subject_graph(graph, subject)
         self.get_name() # Get the name from the subject
 
@@ -120,30 +120,34 @@ class Node:
             graph (Graph, optional): The Graph. Defaults to None.
             subject (URIRef, optional): The Subject. Defaults to None.
         """
-        if(graph): # A new graph is given, overwrite the existing nodeGraph
-            self.graph = graph
-        if(not self.graph):
+        if(not graph):
             raise ValueError("No graph has been assigned to the object, please add one")
 
         if (subject): # A new subject is given, overwrite the existing subject
-            self.subject = subject
+            self.subject = subject #
+
         elif (not self.subject): # No subject is defined yet, pick the first one
             self.subject=next(self.graph.subjects())
         if(self.subject not in self.graph.subjects()): # Evaluate the subject
             raise(ValueError("The provided Subject is not in the Graph"))
 
         newGraph = Graph()
-        newGraph += self.graph.triples((self.subject, None, None))  # Add all the triples of the subject
+        newGraph += graph.triples((self.subject, None, None))  # Add all the triples of the subject
+        self.graph = newGraph
+
         newGraph._set_namespace_manager(self.graph._get_namespace_manager()) # Copy over the full namespace
         self.graph = newGraph # Set the Graph
 
-    def get_subject(self) -> URIRef:
+    def get_subject(self, subject :URIRef = None) -> str:
         """Returns and validates the current subject, if empty, a new one is created based on a unique UID
 
         Returns:
-            URIRef: The subject
+            str: The subject
         """
-        self.subject= ut.get_uriref(getattr(self,'subject',None))
+        if getattr(self,'subject',None) is not None:            
+            self.subject= URIRef(ut.check_string_validity(str(self.subject)))
+        else:
+            self.subject=URIRef(str(uuid.uuid1()))
         return self.subject
 
     def get_name(self) -> str:
@@ -156,31 +160,6 @@ class Node:
             self.name = os.path.basename(os.path.normpath(self.get_subject())) #Get the endpoint of the path removes al the url parts
         return self.name
 
-    def get_folder_path(self) -> str:
-        """Returns the absulute folderPath of this Node
-
-        Raises:
-            ValueError: If neither graphPath or SessionPath is defined
-            FileNotFoundError: If the given graphPath or sessionPath does not lead to a valid systempath
-
-        Returns:
-            str: The Full systempath to its folder
-        """
-
-        if getattr(self, 'graphPath',None) is not None: 
-            folderPath= os.path.dirname(self.graphPath) #The graphpath is the filepath, we need to get a
-        elif getattr(self, 'sessionPath',None) is not None:
-            folderPath=self.sessionPath
-        else:
-            raise ValueError("There is no path defined in this instance")
-
-        # Validate if folderpath actually exists
-        if not os.path.isdir(folderPath): # if a folderpath does not end with a trailing slash, 'dirname' will return the parent directory, so an extra check is needed
-            folderPath = os.path.dirname(folderPath)
-            if not os.path.isdir(folderPath):
-                raise FileNotFoundError("The given path is not a valid folder on this system.")    
-        return folderPath
-
     def get_resource(self):
         """Returns the resource from the Node type. Overwrite this function for each node type
         """
@@ -219,18 +198,18 @@ class Node:
         # return the full path
         return self.fullResourcePath
 
-    def to_graph(self, savePath: str = None):
+    def to_graph(self, graphPath : str = None) -> Graph:
         """Converts the current Node variables to a graph and optionally save it
 
         Args:
-            savePath (str, optional): The full path to wirte the graph to. Defaults to None.
+            graphPath (str, optional): The full path to write the graph to. Defaults to None.
         """
         self.get_subject()  
         # if graph does not exist => create graph
         if not self.graph: 
             self.graph=Graph()              
-            ut.bind_ontologies(self.graph)             
-            self.graph.add((self.get_subject(), RDF.type, Literal(str(type(self)))))
+            ut.bind_ontologies(self.graph)                    
+            self.graph.add((self.subject, RDF.type, Literal(str(type(self)))))
   
         # enumerate attributes in node and write them to triples
         attributes = ut.get_variables_in_class(self)
@@ -246,10 +225,10 @@ class Node:
                 self.graph.add((self.subject, predicate, Literal(str(value))))
             elif value is not None and attribute in pathlist:
                 predicate = ut.match_uri(attribute)
-                folderPath=self.get_folder_path()
-                relpath=os.path.relpath(value,folderPath)
-                self.graph.add((self.subject, predicate, Literal(relpath)))
-
-        if(savePath):
-            ut.save_graph(self.graph, savePath)
+                if (graphPath):
+                    folderPath=ut.get_folder_path(graphPath)
+                    relpath=os.path.relpath(value,folderPath)
+                    self.graph.add((self.subject, predicate, Literal(relpath)))
+                else:
+                    self.graph.add((self.subject, predicate, Literal(value)))
         return self.graph
\ No newline at end of file
