@@ -280,7 +280,7 @@ class PointCloudNode (geomapi.geometrynode.GeometryNode):
                                                 ut.xml_to_float(rotationnode[1].text),
                                                 ut.xml_to_float(rotationnode[2].text) ])
                                 quatArray=quatArray.astype(np.float)   
-                                quatArray=np.nan_to_num(quaternion)                
+                                quatArray=np.nan_to_num(quatArray)                
                             except:
                                 quatArray=np.array([0,0,0,1])
 
